"use strict";
// Copyright (c) Vidar Tonaas Fauske
// Distributed under the terms of the Modified BSD License.
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
/*

This module contains code for opening a new voila view.

*/
const application_1 = require("@jupyterlab/application");
const apputils_1 = require("@jupyterlab/apputils");
const apputilsExtension = __importStar(require("@jupyterlab/apputils-extension"));
const coreutils_1 = require("@jupyterlab/coreutils");
const rendermime_1 = require("@jupyterlab/rendermime");
const base = __importStar(require("@jupyter-widgets/base"));
// We import only the version from the specific module in controls so that the
// controls code can be split and dynamically loaded in webpack.
const version_1 = require("@jupyter-widgets/controls/lib/version");
const output_1 = require("./output");
const jupyterlab_manager_1 = require("@jupyter-widgets/jupyterlab-manager");
const tokens_1 = require("./tokens");
const registry_1 = require("./registry");
const widget_1 = require("./widget");
require("../style/index.css");
const originalResolver = apputilsExtension.default.find(v => v.provides === apputils_1.IWindowResolver);
const WIDGET_VIEW_MIMETYPE = 'application/vnd.jupyter.widget-view+json';
const WIDGET_REGISTRY = new registry_1.WidgetRegistry();
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/base',
    version: base.JUPYTER_WIDGETS_VERSION,
    exports: {
        WidgetModel: base.WidgetModel,
        WidgetView: base.WidgetView,
        DOMWidgetView: base.DOMWidgetView,
        DOMWidgetModel: base.DOMWidgetModel,
        LayoutModel: base.LayoutModel,
        LayoutView: base.LayoutView,
        StyleModel: base.StyleModel,
        StyleView: base.StyleView
    }
});
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/controls',
    version: version_1.JUPYTER_CONTROLS_VERSION,
    exports: () => {
        return new Promise((resolve, reject) => {
            require.ensure(['@jupyter-widgets/controls'], (require) => {
                resolve(require('@jupyter-widgets/controls'));
            }, (err) => {
                reject(err);
            }, '@jupyter-widgets/controls');
        });
    }
});
WIDGET_REGISTRY.register({
    name: '@jupyter-widgets/output',
    version: output_1.OUTPUT_WIDGET_VERSION,
    exports: { OutputModel: output_1.OutputModel, OutputView: output_1.OutputView }
});
const voilaViewPlugin = {
    id: 'phoila:voila-view',
    requires: [rendermime_1.IRenderMimeRegistry],
    optional: [apputils_1.ICommandPalette, application_1.ILayoutRestorer, rendermime_1.ILatexTypesetter],
    provides: tokens_1.TVoilaTracker,
    activate: (app, rendermime, palette, restorer, typesetter) => {
        // Workaround for https://github.com/jupyter-widgets/ipywidgets/issues/2253
        if (typesetter) {
            typesetter.typeset(document.createElement('div'));
        }
        const { commands, shell } = app;
        const tracker = new apputils_1.WidgetTracker({ namespace: 'phoila' });
        commands.addCommand('phoila:open-new', {
            label: 'Open New Voila View',
            execute: args => {
                const path = args['path'];
                let ppromise;
                if (!path || typeof path !== 'string') {
                    ppromise = apputils_1.InputDialog.getText({
                        title: 'Input notebook path to open',
                        placeholder: 'Notebook path'
                    }).then((result) => {
                        if (result.button.accept && result.value) {
                            return result.value;
                        }
                        throw new Error('No valid path');
                    });
                }
                else {
                    ppromise = Promise.resolve(path);
                }
                void ppromise.then((nbPath) => {
                    // Open voila widget
                    const view = new widget_1.VoilaView(nbPath, WIDGET_REGISTRY, rendermime);
                    view.id = apputils_1.DOMUtils.createDomID();
                    view.title.label = nbPath;
                    view.title.closable = true;
                    view.title.iconClass = 'jp-VoilaIcon';
                    shell.add(view);
                    tracker.add(view);
                });
            }
        });
        if (restorer) {
            restorer.restore(tracker, {
                command: 'phoila:open-new',
                args: view => ({
                    path: view.notebookPath,
                }),
                name: (view) => view.notebookPath,
            });
        }
        if (palette) {
            palette.addItem({
                command: 'phoila:open-new',
                category: 'Voila',
            });
        }
        return tracker;
    },
    autoStart: true
};
const singlePattern = /^\/phoila\/single\/([^?]+)/;
const magicKey = 'phoila-single-workspace';
const hideSidebarTabCss = `.p-Widget.p-TabBar.jp-SideBar.jp-mod-left {
  display: none;
  min-width: 0;
}`;
class CustomResolver {
    constructor() {
        this.name = magicKey;
    }
}
/**
 * Ensure we use our custom workspace name for single mode.
 */
const resolver = {
    id: 'phoila:resolver',
    autoStart: true,
    provides: apputils_1.IWindowResolver,
    requires: [application_1.JupyterFrontEnd.IPaths, application_1.IRouter],
    activate: (_, paths, router) => __awaiter(this, void 0, void 0, function* () {
        singlePattern;
        const parsed = coreutils_1.URLExt.parse(window.location.href);
        const path = parsed.pathname.replace(paths.urls.base, '/');
        if (singlePattern.test(path)) {
            return new CustomResolver();
        }
        return originalResolver.activate(_, paths, router);
    })
};
const singleModePlugin = {
    id: 'phoila:single-mode',
    requires: [application_1.IRouter, tokens_1.TVoilaTracker],
    optional: [coreutils_1.IStateDB],
    activate: (app, router) => {
        const { commands } = app;
        commands.addCommand('phoila:open-single', {
            execute: (args) => __awaiter(this, void 0, void 0, function* () {
                const match = args.path.match(singlePattern);
                try {
                    let path = decodeURI(match[1]);
                    yield commands.execute('phoila:open-new', { path });
                }
                catch (error) {
                    console.warn('Single notebook routing failed.', error);
                }
                commands.execute('application:set-mode', { mode: 'single-document' });
                const style = document.createElement('style');
                document.head.appendChild(style);
                style.sheet.insertRule(hideSidebarTabCss);
            })
        });
        router.register({
            command: 'phoila:open-single',
            pattern: singlePattern,
        });
    },
    autoStart: true
};
/**
 * The widget manager provider.
 */
const phoilaWidgetManagerPlugin = {
    id: 'phoila:extended-widget-manager-plugin',
    requires: [rendermime_1.IRenderMimeRegistry],
    provides: tokens_1.TPhoilaWidgetRegistry,
    activate: (app, rendermime) => {
        // Add a placeholder widget renderer.
        rendermime.addFactory({
            safe: false,
            mimeTypes: [WIDGET_VIEW_MIMETYPE],
            createRenderer: options => new jupyterlab_manager_1.WidgetRenderer(options)
        }, 0);
        return {
            registerWidget(data) {
                WIDGET_REGISTRY.register(data);
            }
        };
    },
    autoStart: true
};
/**
 * The widget manager provider.
 */
const standardWidgetManagerPlugin = {
    id: 'phoila:widget-manager-plugin',
    requires: [tokens_1.TPhoilaWidgetRegistry],
    provides: base.IJupyterWidgetRegistry,
    activate: (app, registry) => {
        return registry;
    },
    autoStart: true
};
exports.default = [
    voilaViewPlugin,
    singleModePlugin,
    resolver,
    phoilaWidgetManagerPlugin,
    standardWidgetManagerPlugin
];
//# sourceMappingURL=plugins.js.map