"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __asyncValues = (this && this.__asyncValues) || function (o) {
    if (!Symbol.asyncIterator) throw new TypeError("Symbol.asyncIterator is not defined.");
    var m = o[Symbol.asyncIterator], i;
    return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }
};
Object.defineProperty(exports, "__esModule", { value: true });
const outputarea_1 = require("@jupyterlab/outputarea");
const jupyterlab_manager_1 = require("@jupyter-widgets/jupyterlab-manager");
const algorithm_1 = require("@phosphor/algorithm");
const messaging_1 = require("@phosphor/messaging");
const widgets_1 = require("@phosphor/widgets");
const voila_1 = require("./voila");
const widget_manager_1 = require("./widget-manager");
const WIDGET_VIEW_MIMETYPE = 'application/vnd.jupyter.widget-view+json';
const VOILA_VIEW_CLASS = 'voila-viewWidget';
class ReplaceLayout extends widgets_1.Layout {
    constructor() {
        super(...arguments);
        this._widgets = [];
    }
    replaceNode(target, widget) {
        this._widgets.push(widget);
        // Send a `'before-attach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeAttach);
        }
        // Insert the widget's node before the sibling.
        target.parentElement.insertBefore(widget.node, target);
        // Send an `'after-attach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterAttach);
        }
    }
    removeWidget(widget) {
        this._widgets.splice(this._widgets.indexOf(widget), 1);
        // Send a `'before-detach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.BeforeDetach);
        }
        // Remove the widget's node from the parent.
        widget.node.remove();
        // Send an `'after-detach'` message if the parent is attached.
        if (this.parent.isAttached) {
            messaging_1.MessageLoop.sendMessage(widget, widgets_1.Widget.Msg.AfterDetach);
        }
        if (this.parent) {
            // Post a fit request for the parent widget.
            this.parent.fit();
        }
    }
    iter() {
        return algorithm_1.iter(this._widgets);
    }
}
class VoilaOutputWidget extends widgets_1.Widget {
    constructor(options) {
        super(options);
        this.layout = new ReplaceLayout();
    }
    replaceWidgetOutputs(rendermime) {
        const rootNode = this.node;
        const outputs = rootNode.querySelectorAll('script[type="application/x.voila-lab-output+json"]');
        for (let i = 0; i != outputs.length; ++i) {
            const node = outputs[i];
            try {
                const model = new outputarea_1.OutputAreaModel();
                const data = JSON.parse(node.innerText);
                model.fromJSON(data.outputs);
                model.trusted = true;
                const view = new outputarea_1.OutputArea({
                    model,
                    rendermime: rendermime
                });
                this.layout.replaceNode(node, view);
            }
            catch (error) {
                console.error(error);
                // Each widget view tag rendering is wrapped with a try-catch statement.
                //
                // This fixes issues with widget models that are explicitely "closed"
                // but are still referred to in a previous cell output.
                // Without the try-catch statement, this error interupts the loop and
                // prevents the rendering of further cells.
                //
                // This workaround may not be necessary anymore with templates that make use
                // of progressive rendering.
            }
        }
    }
    *renderers() {
        for (const outputArea of algorithm_1.toArray(this.layout)) {
            for (const codecell of outputArea.widgets) {
                for (const output of algorithm_1.toArray(codecell.children())) {
                    if (output instanceof jupyterlab_manager_1.WidgetRenderer) {
                        yield output;
                    }
                }
            }
        }
    }
}
exports.VoilaOutputWidget = VoilaOutputWidget;
class VoilaView extends widgets_1.Widget {
    /**
     *
     */
    constructor(notebookPath, registry, rendermime) {
        super();
        this.layout = new widgets_1.PanelLayout();
        this.notebookPath = notebookPath;
        this.populateFromPath(notebookPath);
        this.rendermime = rendermime.clone();
        this.registry = registry;
        this.addClass(VOILA_VIEW_CLASS);
    }
    populateFromPath(path) {
        return __awaiter(this, void 0, void 0, function* () {
            const gen = voila_1.requestVoila(path);
            return this.populateFromGenerator(gen);
        });
    }
    populateFromGenerator(generator) {
        var generator_1, generator_1_1;
        var e_1, _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                for (generator_1 = __asyncValues(generator); generator_1_1 = yield generator_1.next(), !generator_1_1.done;) {
                    const entry = generator_1_1.value;
                    if (VoilaView.isHeader(entry)) {
                        // await?
                        this.processHeader(entry);
                    }
                    if (VoilaView.validateEntry(entry)) {
                        this.addEntry(entry);
                    }
                }
            }
            catch (e_1_1) { e_1 = { error: e_1_1 }; }
            finally {
                try {
                    if (generator_1_1 && !generator_1_1.done && (_a = generator_1.return)) yield _a.call(generator_1);
                }
                finally { if (e_1) throw e_1.error; }
            }
        });
    }
    processHeader(header) {
        return __awaiter(this, void 0, void 0, function* () {
            const { kernelId } = header;
            const kernel = yield voila_1.connectKernel(kernelId);
            const wManager = new widget_manager_1.WidgetManager(kernel, this.rendermime);
            this.registry.data.forEach(data => wManager.register(data));
            // Replace the placeholder widget renderer with one bound to this widget
            // manager.
            this.rendermime.removeMimeType(WIDGET_VIEW_MIMETYPE);
            this.rendermime.addFactory({
                safe: false,
                mimeTypes: [WIDGET_VIEW_MIMETYPE],
                createRenderer: (options) => new jupyterlab_manager_1.WidgetRenderer(options, wManager)
            }, 0);
            algorithm_1.each(this.layout, (widget) => {
                for (const renderer of widget.renderers()) {
                    renderer.manager = wManager;
                }
            });
        });
    }
    addEntry(entry) {
        const view = new VoilaOutputWidget();
        view.node.innerHTML = entry.source;
        this.layout.addWidget(view);
        view.replaceWidgetOutputs(this.rendermime);
    }
}
exports.VoilaView = VoilaView;
(function (VoilaView) {
    function validateEntry(entry) {
        return (entry !== null && entry !== undefined
            && Object.keys(entry).indexOf('source') !== -1
            && typeof entry.source === 'string');
    }
    VoilaView.validateEntry = validateEntry;
    function isHeader(entry) {
        return (entry !== null && entry !== undefined
            && Object.keys(entry).indexOf('kernelId') !== -1
            && typeof entry.kernelId === 'string');
    }
    VoilaView.isHeader = isHeader;
})(VoilaView = exports.VoilaView || (exports.VoilaView = {}));
//# sourceMappingURL=widget.js.map